package sun.awt;

import java.awt.Dialog;
import java.applet.Applet;
import java.awt.peer.FramePeer;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.Dimension;
import java.awt.peer.ComponentPeer;
import java.awt.Cursor;
import java.awt.MenuComponent;
import java.awt.MenuBar;
import java.util.List;
import java.awt.Image;
import java.awt.Component;
import java.awt.AWTKeyStroke;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.awt.Container;
import java.awt.KeyboardFocusManager;
import java.beans.PropertyChangeListener;
import java.awt.KeyEventDispatcher;
import java.awt.Frame;

public abstract class EmbeddedFrame extends Frame implements KeyEventDispatcher, PropertyChangeListener
{
    private boolean isCursorAllowed;
    private boolean supportsXEmbed;
    private KeyboardFocusManager appletKFM;
    private static final long serialVersionUID = 2967042741780317130L;
    protected static final boolean FORWARD = true;
    protected static final boolean BACKWARD = false;
    
    public boolean supportsXEmbed() {
        return this.supportsXEmbed && SunToolkit.needsXEmbed();
    }
    
    protected EmbeddedFrame(boolean b) {
        this(0L, b);
    }
    
    protected EmbeddedFrame() {
        this(0L);
    }
    
    @Deprecated
    protected EmbeddedFrame(int n) {
        this((long)n);
    }
    
    protected EmbeddedFrame(long n) {
        this(n, false);
    }
    
    protected EmbeddedFrame(long n, boolean supportsXEmbed) {
        this.isCursorAllowed = true;
        this.supportsXEmbed = false;
        this.supportsXEmbed = supportsXEmbed;
        this.registerListeners();
    }
    
    @Override
    public Container getParent() {
        return null;
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (!propertyChangeEvent.getPropertyName().equals("managingFocus")) {
            return;
        }
        if (propertyChangeEvent.getNewValue() == Boolean.TRUE) {
            return;
        }
        this.removeTraversingOutListeners((KeyboardFocusManager)propertyChangeEvent.getSource());
        this.appletKFM = KeyboardFocusManager.getCurrentKeyboardFocusManager();
        if (this.isVisible()) {
            this.addTraversingOutListeners(this.appletKFM);
        }
        return;
    }
    
    private void addTraversingOutListeners(KeyboardFocusManager keyboardFocusManager) {
        keyboardFocusManager.addKeyEventDispatcher(this);
        keyboardFocusManager.addPropertyChangeListener("managingFocus", this);
    }
    
    private void removeTraversingOutListeners(KeyboardFocusManager keyboardFocusManager) {
        keyboardFocusManager.removeKeyEventDispatcher(this);
        keyboardFocusManager.removePropertyChangeListener("managingFocus", this);
    }
    
    public void registerListeners() {
        if (this.appletKFM != null) {
            this.removeTraversingOutListeners(this.appletKFM);
        }
        this.appletKFM = KeyboardFocusManager.getCurrentKeyboardFocusManager();
        if (this.isVisible()) {
            this.addTraversingOutListeners(this.appletKFM);
        }
    }
    
    @Override
    public void show() {
        if (this.appletKFM != null) {
            this.addTraversingOutListeners(this.appletKFM);
        }
        super.show();
    }
    
    @Override
    public void hide() {
        if (this.appletKFM != null) {
            this.removeTraversingOutListeners(this.appletKFM);
        }
        super.hide();
    }
    
    @Override
    public boolean dispatchKeyEvent(KeyEvent anEvent) {
        if (this != AWTAccessor.getKeyboardFocusManagerAccessor().getCurrentFocusCycleRoot()) {
            return false;
        }
        if (anEvent.getID() == 400) {
            return false;
        }
        if (!this.getFocusTraversalKeysEnabled() || anEvent.isConsumed()) {
            return false;
        }
        AWTKeyStroke awtKeyStrokeForEvent = AWTKeyStroke.getAWTKeyStrokeForEvent(anEvent);
        Component component = anEvent.getComponent();
        if (this.getFocusTraversalKeys(0).contains(awtKeyStrokeForEvent)) {
            Component lastComponent = this.getFocusTraversalPolicy().getLastComponent(this);
            if ((component == lastComponent || lastComponent == null) && this.traverseOut(true)) {
                anEvent.consume();
                return true;
            }
        }
        if (this.getFocusTraversalKeys(1).contains(awtKeyStrokeForEvent)) {
            Component firstComponent = this.getFocusTraversalPolicy().getFirstComponent(this);
            if ((component == firstComponent || firstComponent == null) && this.traverseOut(false)) {
                anEvent.consume();
                return true;
            }
        }
        return false;
    }
    
    public boolean traverseIn(boolean b) {
        Component component;
        if (b) {
            component = this.getFocusTraversalPolicy().getFirstComponent(this);
        }
        else {
            component = this.getFocusTraversalPolicy().getLastComponent(this);
        }
        if (component != null) {
            AWTAccessor.getKeyboardFocusManagerAccessor().setMostRecentFocusOwner(this, component);
            this.synthesizeWindowActivation(true);
        }
        return null != component;
    }
    
    protected boolean traverseOut(boolean b) {
        return false;
    }
    
    @Override
    public void setTitle(String s) {
    }
    
    @Override
    public void setIconImage(Image image) {
    }
    
    @Override
    public void setIconImages(List<? extends Image> list) {
    }
    
    @Override
    public void setMenuBar(MenuBar menuBar) {
    }
    
    @Override
    public void setResizable(boolean b) {
    }
    
    @Override
    public void remove(MenuComponent menuComponent) {
    }
    
    @Override
    public boolean isResizable() {
        return true;
    }
    
    @Override
    public void addNotify() {
        synchronized (this.getTreeLock()) {
            if (this.getPeer() == null) {
                this.setPeer(new EmbeddedFrame.NullEmbeddedFramePeer());
            }
            super.addNotify();
        }
    }
    
    public void setCursorAllowed(boolean isCursorAllowed) {
        this.isCursorAllowed = isCursorAllowed;
        this.getPeer().updateCursorImmediately();
    }
    
    public boolean isCursorAllowed() {
        return this.isCursorAllowed;
    }
    
    @Override
    public Cursor getCursor() {
        return this.isCursorAllowed ? super.getCursor() : Cursor.getPredefinedCursor(0);
    }
    
    protected void setPeer(ComponentPeer componentPeer) {
        AWTAccessor.getComponentAccessor().setPeer(this, componentPeer);
    }
    
    public void synthesizeWindowActivation(boolean b) {
    }
    
    protected void setLocationPrivate(int n, int n2) {
        Dimension size = this.getSize();
        this.setBoundsPrivate(n, n2, size.width, size.height);
    }
    
    protected Point getLocationPrivate() {
        Rectangle boundsPrivate = this.getBoundsPrivate();
        return new Point(boundsPrivate.x, boundsPrivate.y);
    }
    
    protected void setBoundsPrivate(int n, int n2, int n3, int n4) {
        FramePeer framePeer = (FramePeer)this.getPeer();
        if (framePeer != null) {
            framePeer.setBoundsPrivate(n, n2, n3, n4);
        }
    }
    
    protected Rectangle getBoundsPrivate() {
        FramePeer framePeer = (FramePeer)this.getPeer();
        if (framePeer != null) {
            return framePeer.getBoundsPrivate();
        }
        return this.getBounds();
    }
    
    @Override
    public void toFront() {
    }
    
    @Override
    public void toBack() {
    }
    
    public abstract void registerAccelerator(AWTKeyStroke p0);
    
    public abstract void unregisterAccelerator(AWTKeyStroke p0);
    
    public static Applet getAppletIfAncestorOf(Component component) {
        Container container = component.getParent();
        Applet applet = null;
        while (container != null && !(container instanceof EmbeddedFrame)) {
            if ((container instanceof Applet)) {
                applet = (Applet)container;
            }
            container = container.getParent();
        }
        return (container == null) ? null : applet;
    }
    
    public void notifyModalBlocked(Dialog dialog, boolean b) {
    }
    
    private static class NullEmbeddedFramePeer extends NullComponentPeer implements FramePeer
    {
        @Override
        public void setTitle(String s) {
        }
        
        public void setIconImage(Image image) {
        }
        
        @Override
        public void updateIconImages() {
        }
        
        @Override
        public void setMenuBar(MenuBar menuBar) {
        }
        
        @Override
        public void setResizable(boolean b) {
        }
        
        @Override
        public void setState(int n) {
        }
        
        @Override
        public int getState() {
            return 0;
        }
        
        @Override
        public void setMaximizedBounds(Rectangle rectangle) {
        }
        
        @Override
        public void toFront() {
        }
        
        @Override
        public void toBack() {
        }
        
        @Override
        public void updateFocusableWindowState() {
        }
        
        public void updateAlwaysOnTop() {
        }
        
        @Override
        public void updateAlwaysOnTopState() {
        }
        
        public Component getGlobalHeavyweightFocusOwner() {
            return null;
        }
        
        @Override
        public void setBoundsPrivate(int n, int n2, int n3, int n4) {
            this.setBounds(n, n2, n3, n4, 3);
        }
        
        @Override
        public Rectangle getBoundsPrivate() {
            return this.getBounds();
        }
        
        @Override
        public void setModalBlocked(Dialog dialog, boolean b) {
        }
        
        public void restack() {
            throw new UnsupportedOperationException();
        }
        
        public boolean isRestackSupported() {
            return false;
        }
        
        public boolean requestWindowFocus() {
            return false;
        }
        
        @Override
        public void updateMinimumSize() {
        }
        
        @Override
        public void setOpacity(float n) {
        }
        
        @Override
        public void setOpaque(boolean b) {
        }
        
        @Override
        public void updateWindow() {
        }
        
        @Override
        public void repositionSecurityWarning() {
        }
        
        @Override
        public void emulateActivation(boolean b) {
        }
    }
}
